package org.div8.dictionary;

import java.io.IOException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

public class DefinitionView extends Composite {
	class DictLookupThread extends Thread {
		String lookupWord;
		String definition;
		DictionarySettings settings;
		boolean done = false;

		DictLookupThread(String threadname) {
			super(threadname);
		}
		public void performLookup(DictionarySettings dictsettings, String word) {
			lookupWord = word;
			settings = dictsettings;
			start();
		}
		public boolean isDone() { return done; }
		public String getDefinition() { return definition; }
		public void run() {
			try {
				DictClientSocket dc = new DictClientSocket(settings.server, settings.portnum);
				definition = dc.getDefinition(lookupWord);
				dc.shutDown();
			} catch (IOException e) {
				definition = "Error looking up \"" + lookupWord + "\"";
			}
			done = true;
		}
	}

	DictionarySettings settings;
	Text wordEntry;
	Text definition;
	DictLookupThread activeLookup = null;
	final int waitTime = 500;

	class DictionaryWaiter implements Runnable {
		DefinitionView view;
		public void run() {
			if(view.activeLookup.isDone()) {
				view.definition.setText(view.activeLookup.getDefinition());
				view.activeLookup = null;
			} else {
				getDisplay().timerExec(waitTime, this);
			}
		}
	}

	void lookupWord() {
		DictionaryWaiter waiter = new DictionaryWaiter();
		waiter.view = this;
		String word = wordEntry.getText();
		wordEntry.setText("");
		definition.setText("Looking up \"" + word + "\"");
		activeLookup = new DictLookupThread("Dictionary Lookup");
		activeLookup.performLookup(settings, word);
		getDisplay().timerExec(waitTime, waiter);
	}

	DefinitionView(DictionarySettings dictsettings, Composite parent, int style) {
		super(parent, style);

		settings = dictsettings;

		// Lay out the window.
		setLayout(new GridLayout());

		Composite topRow = new Composite(this, SWT.NONE);
		GridLayout topLayout = new GridLayout(2, false);
		topLayout.marginHeight = 0;
		topLayout.marginWidth = 0;
		topRow.setLayout(topLayout);
		topRow.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// A text entry.
		wordEntry = new Text(topRow, SWT.BORDER);
		wordEntry.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		wordEntry.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if( e.character == '\n' || e.character == '\r' ) {
					lookupWord();
				}
			}
		});

		// Text box for the definition.
		definition = new Text(this, SWT.BORDER | SWT.MULTI);
		definition.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL));

		// Hack: Make the definition box appear editable, but make sure it isn't.
		Color fg = definition.getForeground();
		Color bg = definition.getBackground();
		definition.setEditable(false);
		definition.setBackground(bg);
		definition.setForeground(fg);

		// A button
		Button lookup = new Button(topRow, SWT.PUSH);
		lookup.setText("Look up");
		lookup.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				lookupWord();
			}
		});		
	}
}
